/*
GNU General Public License

Archichronos - Modern Art Clock
Copyright (C) 2000  Howard A Kistler

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

package com.hexidec.tool;

import java.awt.AlphaComposite;
import java.awt.BasicStroke;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Font;
import java.awt.Frame;
import java.awt.GradientPaint;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Insets;
import java.awt.Paint;
import java.awt.Panel;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.image.BufferedImage;
import java.util.Calendar;
import java.util.Random;

/** Archichronos
  *
  * @author Howard Kistler
  * @version 0.1
  *
  * VERSION HISTORY
  * 0.1 (11/03/2000) - initial creation (10/31/2000)
  */

public class Archichronos extends Frame implements Runnable {

	private int clockWidth  = 200;
	private int clockHeight = 200;

	private Panel pnlClock;
	private BufferedImage bufferAlpha;
	private long oldTime = System.currentTimeMillis();

	// Display Flags
	private boolean showNumericTime = true;
	private boolean showAsRings     = true;
	private boolean showHourHand    = true;
	private boolean showMinuteHand  = true;
	private boolean showSecondHand  = true;
	private boolean showMillisHand  = false;
	private boolean showFasterInner = true;
	private boolean showArcTrim     = false;

	// Translucency Flags
	private boolean transHour       = false;
	private boolean transMinute     = false;
	private boolean transSecond     = true;
	private boolean transMillis     = true;

	// Gradient Paint Flags
	private boolean gradientHour    = true;
	private boolean gradientMinute  = true;
	private boolean gradientSecond  = false;
	private boolean gradientMillis  = false;

	// Random Colors Flags
	private boolean randomHour    = true;
	private boolean randomMinute  = true;
	private boolean randomSecond  = false;
	private boolean randomMillis  = false;

	// Base Colors
	private Color colorBack   = Color.black;
	private Color colorTrim   = Color.gray;
	private Color colorHour   = Color.blue;
	private Color colorMinute = Color.gray;
	private Color colorSecond = Color.white;
	private Color colorMillis = Color.red;

	private int colorHourAlphaR = 128;
	private int colorHourAlphaG = 128;
	private int colorHourAlphaB = 255;
	private int colorHourBetaR  = 255;
	private int colorHourBetaG  = 255;
	private int colorHourBetaB  = 128;
	private int colorMinuteAlphaR = 255;
	private int colorMinuteAlphaG = 128;
	private int colorMinuteAlphaB = 128;
	private int colorMinuteBetaR  = 128;
	private int colorMinuteBetaG  = 255;
	private int colorMinuteBetaB  = 128;

	public Archichronos(boolean bN, boolean bB, boolean bR, boolean bI, boolean bH, boolean bM, boolean bS, boolean bU)
	{
		super("Archichronos");

		this.addWindowListener(
			new WindowAdapter()
			{
				public void windowClosing(WindowEvent we)
				{
					System.exit(0);
				}
			}
		);

		showNumericTime = bN;
		showAsRings     = bR;
		showArcTrim     = bB;
		showFasterInner = bI;
		showHourHand    = bH;
		showMinuteHand  = bM;
		showSecondHand  = bS;
		showMillisHand  = bU;

		pnlClock = new Panel();
		pnlClock.setSize(clockWidth, clockHeight);
		pnlClock.setBounds(0, 0, clockWidth, clockHeight);

		bufferAlpha = (BufferedImage)(createImage(clockWidth, clockHeight));

		this.setLayout(new BorderLayout());
		this.add(pnlClock, BorderLayout.CENTER);
		Insets insets = this.getInsets();
		this.setSize(clockWidth + insets.left + insets.right, clockHeight + insets.top + insets.bottom);
		this.show();

		run();
	}

	public void run()
	{
		while(true)
		{
			if(oldTime != System.currentTimeMillis())
			{
				paint(this.getGraphics());
				oldTime = System.currentTimeMillis();
			}
		}
	}

	private void paintHand(Graphics2D graphicsObject, Paint paintObject, boolean isTranslucent, int arcStartX, int arcStartY, int arcWidth, int arcHeight, int arcAngle)
	{
		graphicsObject.setPaint(paintObject);
		if(isTranslucent)
		{
			graphicsObject.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 0.5f));
		}
		graphicsObject.fillArc(arcStartX, arcStartY, arcWidth, arcHeight, 90, -arcAngle);
		graphicsObject.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 1.0f));
		if(showArcTrim)
		{
			graphicsObject.setPaint(colorTrim);
			graphicsObject.drawArc(arcStartX, arcStartY, arcWidth, arcHeight, 90, -arcAngle);
		}
	}

	public void paint(Graphics g)
	{
		if(bufferAlpha == null)
		{
			bufferAlpha = (BufferedImage)(createImage(clockWidth, clockHeight));
		}
		Calendar calen = Calendar.getInstance();
		Graphics2D g2D = (Graphics2D)(bufferAlpha.getGraphics());
		Graphics2D g2Dpanel = (Graphics2D)(pnlClock.getGraphics());
		g2D.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
		g2D.setStroke(new BasicStroke(2.0f, BasicStroke.CAP_ROUND, BasicStroke.JOIN_MITER));

		// clear background
		g2D.setPaint(colorBack);
		g2D.fill(new Rectangle(0, 0, clockWidth, clockHeight));

		Paint gradPaint;

		int calenHour12 = calen.get(Calendar.HOUR);
		if(calenHour12 == 0) { calenHour12 = 12; }

		if(showFasterInner)
		{
			if(showHourHand)
			{
				Color colorAlpha = new Color(colorHourAlphaR, colorHourAlphaG, colorHourAlphaB);
				Color colorBeta  = new Color(colorHourBetaR, colorHourBetaG, colorHourBetaB);
				gradPaint = new GradientPaint(100.0f, 30.0f, colorAlpha, 100.0f, 140.0f, colorBeta);
				paintHand(g2D, gradPaint, transHour, 30, 30, 140, 140, (calen.get(Calendar.HOUR) * 30));
				if(randomHour)
				{
					Random rnd = new Random(System.currentTimeMillis());
					int which = rnd.nextInt(6);
					int offset = rnd.nextInt(2);
					if(offset == 0) { offset = -1; }
					if(which == 0)      { colorHourAlphaR = colorHourAlphaR + offset; }
					else if(which == 1) { colorHourAlphaG = colorHourAlphaG + offset; }
					else if(which == 2) { colorHourAlphaB = colorHourAlphaB + offset; }
					else if(which == 4) { colorHourBetaR  = colorHourBetaR  + offset; }
					else if(which == 5) { colorHourBetaG  = colorHourBetaG  + offset; }
					else if(which == 6) { colorHourBetaB  = colorHourBetaB  + offset; }
					if(colorHourAlphaR < 0)   { colorHourAlphaR = 0; }
					if(colorHourAlphaG < 0)   { colorHourAlphaG = 0; }
					if(colorHourAlphaB < 0)   { colorHourAlphaB = 0; }
					if(colorHourBetaR  < 0)   { colorHourBetaR  = 0; }
					if(colorHourBetaG  < 0)   { colorHourBetaG  = 0; }
					if(colorHourBetaB  < 0)   { colorHourBetaB  = 0; }
					if(colorHourAlphaR > 255) { colorHourAlphaR = 255; }
					if(colorHourAlphaG > 255) { colorHourAlphaG = 255; }
					if(colorHourAlphaB > 255) { colorHourAlphaB = 255; }
					if(colorHourBetaR  > 255) { colorHourBetaR  = 255; }
					if(colorHourBetaG  > 255) { colorHourBetaG  = 255; }
					if(colorHourBetaB  > 255) { colorHourBetaB  = 255; }
				}
			}
			if(showMinuteHand)
			{
				Color colorAlpha = new Color(colorMinuteAlphaR, colorMinuteAlphaG, colorMinuteAlphaB);
				Color colorBeta  = new Color(colorMinuteBetaR, colorMinuteBetaG, colorMinuteBetaB);
				gradPaint = new GradientPaint(50.0f, 100.0f, colorAlpha, 150.0f, 100.0f, colorBeta);
				paintHand(g2D, gradPaint, transMinute, 40, 40, 120, 120, (calen.get(Calendar.MINUTE) * 6));
				if(randomMinute)
				{
					Random rnd = new Random(System.currentTimeMillis());
					int which = rnd.nextInt(6);
					int offset = rnd.nextInt(2);
					if(offset == 0) { offset = -1; }
					if(which == 0)      { colorMinuteAlphaR = colorMinuteAlphaR + offset; }
					else if(which == 1) { colorMinuteAlphaG = colorMinuteAlphaG + offset; }
					else if(which == 2) { colorMinuteAlphaB = colorMinuteAlphaB + offset; }
					else if(which == 4) { colorMinuteBetaR  = colorMinuteBetaR  + offset; }
					else if(which == 5) { colorMinuteBetaG  = colorMinuteBetaG  + offset; }
					else if(which == 6) { colorMinuteBetaB  = colorMinuteBetaB  + offset; }
					if(colorMinuteAlphaR < 0)   { colorMinuteAlphaR = 0; }
					if(colorMinuteAlphaG < 0)   { colorMinuteAlphaG = 0; }
					if(colorMinuteAlphaB < 0)   { colorMinuteAlphaB = 0; }
					if(colorMinuteBetaR  < 0)   { colorMinuteBetaR  = 0; }
					if(colorMinuteBetaG  < 0)   { colorMinuteBetaG  = 0; }
					if(colorMinuteBetaB  < 0)   { colorMinuteBetaB  = 0; }
					if(colorMinuteAlphaR > 255) { colorMinuteAlphaR = 255; }
					if(colorMinuteAlphaG > 255) { colorMinuteAlphaG = 255; }
					if(colorMinuteAlphaB > 255) { colorMinuteAlphaB = 255; }
					if(colorMinuteBetaR  > 255) { colorMinuteBetaR  = 255; }
					if(colorMinuteBetaG  > 255) { colorMinuteBetaG  = 255; }
					if(colorMinuteBetaB  > 255) { colorMinuteBetaB  = 255; }
				}
			}
			if(showSecondHand)
			{
				gradPaint = new GradientPaint(100.0f, 0.0f, Color.cyan, 100.0f, (float)clockHeight, Color.magenta);
				paintHand(g2D, colorSecond, transSecond, 50, 50, 100, 100, (calen.get(Calendar.SECOND) * 6));
			}
			if(showMillisHand)
			{
				gradPaint = new GradientPaint(0.0f, 100.0f, Color.white, (float)clockWidth, 100.0f, Color.gray);
				paintHand(g2D, colorMillis, transMillis, 55, 55, 90, 90, ((int)(calen.get(Calendar.MILLISECOND) * 0.36)));
			}
		}
		else
		{
			if(showMillisHand)
			{
				gradPaint = new GradientPaint(0.0f, 100.0f, Color.white, (float)clockWidth, 100.0f, Color.gray);
				paintHand(g2D, colorMillis, transMillis, 30, 30, 140, 140, ((int)(calen.get(Calendar.MILLISECOND) * 0.36)));
			}
			if(showSecondHand)
			{
				gradPaint = new GradientPaint(100.0f, 0.0f, Color.cyan, 100.0f, (float)clockHeight, Color.magenta);
				if(showMillisHand)
				{
					paintHand(g2D, colorSecond, transSecond, 35, 35, 130, 130, (calen.get(Calendar.SECOND) * 6));
				}
				else
				{
					paintHand(g2D, colorSecond, transSecond, 30, 30, 140, 140, (calen.get(Calendar.SECOND) * 6));
				}
			}
			if(showMinuteHand)
			{
				gradPaint = new GradientPaint(0.0f, 100.0f, Color.red, (float)clockWidth, 100.0f, Color.green);
				paintHand(g2D, gradPaint, transMinute, 40, 40, 120, 120, (calen.get(Calendar.MINUTE) * 6));
			}
			if(showHourHand)
			{
				gradPaint = new GradientPaint(100.0f, 0.0f, Color.blue, 100.0f, (float)clockHeight, Color.yellow);
				paintHand(g2D, gradPaint, transHour, 50, 50, 100, 100, (calen.get(Calendar.HOUR) * 30));
			}
		}

		// paint inner disc
		if(showAsRings)
		{
			g2D.setPaint(colorBack);
			g2D.fillArc(60, 60, 80, 80, 90, 360);
		}

		// paint time string
		if(showNumericTime)
		{
			g2D.setPaint(Color.red);
			String strTime = (calenHour12 < 10 ? "0" : "") + calenHour12 + ":" + (calen.get(Calendar.MINUTE) < 10 ? "0" : "") + calen.get(Calendar.MINUTE) + (showSecondHand ? ":" + (calen.get(Calendar.SECOND) < 10 ? "0" : "") + calen.get(Calendar.SECOND) : "");
			g2D.setFont(new Font("monospaced", Font.BOLD, 14));
			g2D.drawString(strTime, 64, 104);
		}

		// paint buffered image to visible canvas
		g2Dpanel.drawImage(bufferAlpha, 0, 0, this);
	}

	public static void main(String[] args)
	{
		boolean bNumeric = true;
		boolean bBorder  = false;
		boolean bRings   = true;
		boolean bInner   = true;
		boolean bHour    = true;
		boolean bMinute  = true;
		boolean bSecond  = true;
		boolean bMillis  = false;
		for(int i = 0; i < args.length; i++)
		{
			/*
				-T/-t Show/Hide numeric time (Show by default)
				-B/-b Show/Hide border trim (Hide by default)
				-R/-D Show hands as rings/discs (Rings by default)
				-I/-i Show faster time units on the inside/outside (Inside by default)
				-H/-h Show/Hide hour hand (Show by default)
				-M/-m Show/Hide minute hand (Show by default)
				-S/-s Show/Hide second hand (Show by default)
				-U/-u Show/Hide millisecond hand (Hide by default)
			*/
			if(args[i].equals("-T"))      { bNumeric = true;  }
			else if(args[i].equals("-t")) { bNumeric = false; }
			else if(args[i].equals("-B")) { bBorder  = true;  }
			else if(args[i].equals("-b")) { bBorder  = false; }
			else if(args[i].equals("-R")) { bRings   = true;  }
			else if(args[i].equals("-D")) { bRings   = false; }
			else if(args[i].equals("-I")) { bInner   = true;  }
			else if(args[i].equals("-i")) { bInner   = false; }
			else if(args[i].equals("-H")) { bHour    = true;  }
			else if(args[i].equals("-h")) { bHour    = false; }
			else if(args[i].equals("-M")) { bMinute  = true;  }
			else if(args[i].equals("-m")) { bMinute  = false; }
			else if(args[i].equals("-S")) { bSecond  = true;  }
			else if(args[i].equals("-s")) { bSecond  = false; }
			else if(args[i].equals("-U")) { bMillis  = true;  }
			else if(args[i].equals("-u")) { bMillis  = false; }
		}
		Archichronos Archichronos = new Archichronos(bNumeric, bBorder, bRings, bInner, bHour, bMinute, bSecond, bMillis);
	}

}
